@@ -1,4 +1,7 @@
-"""Command registry and factory for TunaCode CLI commands."""
+"""Command registry and factory for TunaCode CLI commands.
+
+CLAUDE_ANCHOR[command-registry]: Central command registration and execution
+"""
 
 from dataclasses import dataclass
 from typing import Any, Dict, List, Optional, Type
@@ -20,6 +23,7 @@ from .implementations.debug import (
 )
 from .implementations.development import BranchCommand, InitCommand
 from .implementations.model import ModelCommand
+from .implementations.plan import ExitPlanCommand, PlanCommand
 from .implementations.system import (
     ClearCommand,
     HelpCommand,
@@ -126,6 +130,8 @@ class CommandRegistry:
             InitCommand,
             TemplateCommand,
             TodoCommand,
+            PlanCommand,  # Add plan command
+            ExitPlanCommand,  # Add exit plan command
         ]
 
         # Register all discovered commands
