@@ -7,16 +7,8 @@ Handles configuration paths, model registries, and application metadata.
 
 from pathlib import Path
 
-from tunacode.constants import (
-    APP_NAME,
-    APP_VERSION,
-    CONFIG_FILE_NAME,
-    TOOL_READ_FILE,
-    TOOL_RUN_COMMAND,
-    TOOL_UPDATE_FILE,
-    TOOL_WRITE_FILE,
-)
-from tunacode.types import ConfigFile, ConfigPath, ToolName
+from tunacode.constants import APP_NAME, APP_VERSION, CONFIG_FILE_NAME, ToolName
+from tunacode.types import ConfigFile, ConfigPath
 
 
 class PathConfig:
@@ -32,8 +24,13 @@ class ApplicationSettings:
         self.guide_file = f"{self.name.upper()}.md"
         self.paths = PathConfig()
         self.internal_tools: list[ToolName] = [
-            TOOL_READ_FILE,
-            TOOL_RUN_COMMAND,
-            TOOL_UPDATE_FILE,
-            TOOL_WRITE_FILE,
+            ToolName.BASH,
+            ToolName.GLOB,
+            ToolName.GREP,
+            ToolName.LIST_DIR,
+            ToolName.READ_FILE,
+            ToolName.RUN_COMMAND,
+            ToolName.TODO,
+            ToolName.UPDATE_FILE,
+            ToolName.WRITE_FILE,
         ]
