@@ -19,6 +19,8 @@ class TestAgentCreation:
         self.state_manager.session = Mock()
         self.state_manager.session.agents = {}
         self.state_manager.session.user_config = {"settings": {"max_retries": 3}}
+        self.state_manager.is_plan_mode = Mock(return_value=False)
+        self.state_manager.todos = []
 
     def test_get_or_create_agent_first_time(self):
         """Capture behavior when creating agent for first time."""
@@ -56,7 +58,7 @@ class TestAgentCreation:
                         call_kwargs = mock_agent_class.call_args.kwargs
                         assert call_kwargs["model"] == model
                         assert call_kwargs["system_prompt"] == system_prompt
-                        assert len(call_kwargs["tools"]) == 9  # All 9 tools registered
+                        assert len(call_kwargs["tools"]) == 10  # All 10 tools registered
                         assert call_kwargs["mcp_servers"] == []
 
     def test_get_or_create_agent_cached(self):
@@ -102,7 +104,7 @@ class TestAgentCreation:
 
                         # Assert - Golden master
                         call_kwargs = mock_agent_class.call_args.kwargs
-                        assert call_kwargs["system_prompt"] == fallback_prompt
+                        assert call_kwargs["system_prompt"] == "You are a helpful AI assistant."
 
     def test_get_or_create_agent_default_prompt(self):
         """Capture behavior when neither prompt file exists."""
@@ -127,10 +129,7 @@ class TestAgentCreation:
 
                         # Assert - Golden master
                         call_kwargs = mock_agent_class.call_args.kwargs
-                        assert (
-                            call_kwargs["system_prompt"]
-                            == "You are a helpful AI assistant for software development tasks."
-                        )
+                        assert call_kwargs["system_prompt"] == "You are a helpful AI assistant."
 
     def test_get_or_create_agent_tools_registered(self):
         """Capture behavior of tool registration with max_retries."""
@@ -156,8 +155,8 @@ class TestAgentCreation:
                     call_kwargs = mock_agent_class.call_args.kwargs
                     tools = call_kwargs["tools"]
 
-                    # Verify all 9 tools are registered
-                    assert len(tools) == 9
+                    # Verify all 10 tools are registered
+                    assert len(tools) == 10
 
                     # Verify each tool has max_retries set
                     for tool in tools:
