@@ -3,7 +3,7 @@ Characterization tests for TunaCode UI async UI updates.
 Covers: async output functions and async-safe UI flows.
 """
 
-from unittest.mock import patch
+from unittest.mock import AsyncMock, patch
 
 import pytest
 
@@ -25,6 +25,8 @@ async def test_async_print_calls_console_print():
 
 @pytest.mark.asyncio
 async def test_async_info_formats_and_prints():
-    with patch("tunacode.ui.output.print") as mock_print:
+    # info() now goes through ui_logger, let's test that it's called
+    with patch("tunacode.ui.output.ui_logger") as mock_logger:
+        mock_logger.info = AsyncMock()
         await output_mod.info("Test info")
-        mock_print.assert_called_once()
+        mock_logger.info.assert_called_once_with("Test info")
